@@ -1,4 +1,4 @@
-# Copyright (c) Microsoft Corporation. All rights reserved.
+# Copyright (c) Microsoft Corporation and Fairlearn contributors.
 # Licensed under the MIT License.
 
 import logging
@@ -7,16 +7,20 @@ import pandas as pd
 import pytest
 from sklearn.linear_model import LogisticRegression, LinearRegression
 
+
 from sklearn.exceptions import NotFittedError
-from fairlearn import _NO_PREDICT_BEFORE_FIT
+from sklearn.pipeline import Pipeline
+from sklearn.preprocessing import StandardScaler
+
 from fairlearn._input_validation import \
     (_MESSAGE_Y_NONE,
      _LABELS_NOT_0_1_ERROR_MESSAGE)
-from fairlearn.reductions import GridSearch, DemographicParity, EqualizedOdds, GroupLossMoment, \
+from fairlearn.reductions import GridSearch, DemographicParity, EqualizedOdds, BoundedGroupLoss, \
     ZeroOneLoss
 from fairlearn.reductions._grid_search._grid_generator import GRID_DIMENSION_WARN_THRESHOLD, \
     GRID_DIMENSION_WARN_TEMPLATE, GRID_SIZE_WARN_TEMPLATE
 
+from test.unit.fixes import get_sklearn_expected_1d_message
 from test.unit.input_convertors import conversions_for_1d, ensure_ndarray, ensure_dataframe
 from test.unit.reductions.conftest import is_invalid_transformation
 from test.unit.reductions.grid_search.utilities import assert_n_grid_search_results, _quick_data
@@ -29,6 +33,11 @@ candidate_X_transforms = [ensure_ndarray, ensure_dataframe]
 candidate_Y_transforms = conversions_for_1d
 candidate_A_transforms = conversions_for_1d
 
+# ==============================================================
+
+not_fitted_error_msg = "This {} instance is not fitted yet. Call 'fit' with " \
+    "appropriate arguments before using this estimator."
+
 
 # Base class for tests
 # Tests which must be passed by all calls to the GridSearch
@@ -40,7 +49,8 @@ class ArgumentTests:
     @pytest.mark.parametrize("A_two_dim", [False, True])
     @pytest.mark.uncollect_if(func=is_invalid_transformation)
     def test_valid_inputs(self, transformX, transformY, transformA, A_two_dim):
-        gs = GridSearch(self.estimator, self.disparity_criterion, grid_size=2)
+        gs = GridSearch(self.estimator, self.disparity_criterion, grid_size=2,
+                        sample_weight_name=self.sample_weight_name)
         X, Y, A = _quick_data(A_two_dim)
         gs.fit(transformX(X),
                transformY(Y),
@@ -54,7 +64,8 @@ class ArgumentTests:
     @pytest.mark.parametrize("A_two_dim", [False, True])
     @pytest.mark.uncollect_if(func=is_invalid_transformation)
     def test_X_is_None(self, transformY, transformA, A_two_dim):
-        gs = GridSearch(self.estimator, self.disparity_criterion, grid_size=3)
+        gs = GridSearch(self.estimator, self.disparity_criterion, grid_size=3,
+                        sample_weight_name=self.sample_weight_name)
         _, Y, A = _quick_data(A_two_dim)
 
         with pytest.raises(ValueError) as execInfo:
@@ -69,7 +80,8 @@ class ArgumentTests:
     @pytest.mark.parametrize("A_two_dim", [False, True])
     @pytest.mark.uncollect_if(func=is_invalid_transformation)
     def test_Y_is_None(self, transformX, transformA, A_two_dim):
-        gs = GridSearch(self.estimator, self.disparity_criterion)
+        gs = GridSearch(self.estimator, self.disparity_criterion,
+                        sample_weight_name=self.sample_weight_name)
         X, _, A = _quick_data()
 
         with pytest.raises(ValueError) as execInfo:
@@ -87,7 +99,8 @@ class ArgumentTests:
     @pytest.mark.parametrize("A_two_dim", [False, True])
     @pytest.mark.uncollect_if(func=is_invalid_transformation)
     def test_X_Y_different_rows(self, transformX, transformY, transformA, A_two_dim):
-        gs = GridSearch(self.estimator, self.disparity_criterion)
+        gs = GridSearch(self.estimator, self.disparity_criterion,
+                        sample_weight_name=self.sample_weight_name)
         X, _, A = _quick_data()
         Y = np.random.randint(2, size=len(A)+1)
 
@@ -105,7 +118,8 @@ class ArgumentTests:
     @pytest.mark.parametrize("A_two_dim", [False, True])
     @pytest.mark.uncollect_if(func=is_invalid_transformation)
     def test_X_A_different_rows(self, transformX, transformY, transformA, A_two_dim):
-        gs = GridSearch(self.estimator, self.disparity_criterion)
+        gs = GridSearch(self.estimator, self.disparity_criterion,
+                        sample_weight_name=self.sample_weight_name)
         X, Y, _ = _quick_data(A_two_dim)
         A = np.random.randint(2, size=len(Y)+1)
         if A_two_dim:
@@ -131,7 +145,8 @@ class ArgumentTests:
         # The purpose of this test case is to create enough groups to trigger certain expected
         # warnings. The scenario should still work and succeed.
         grid_size = 10
-        gs = GridSearch(self.estimator, self.disparity_criterion, grid_size=grid_size)
+        gs = GridSearch(self.estimator, self.disparity_criterion, grid_size=grid_size,
+                        sample_weight_name=self.sample_weight_name)
         X, Y, A = _quick_data(A_two_dim)
 
         if A_two_dim:
@@ -192,7 +207,8 @@ class ArgumentTests:
                         'dimensionality.')
 
         grid_size = 10
-        gs = GridSearch(self.estimator, self.disparity_criterion, grid_size=grid_size)
+        gs = GridSearch(self.estimator, self.disparity_criterion, grid_size=grid_size,
+                        sample_weight_name=self.sample_weight_name)
         X, Y, A = _quick_data(A_two_dim, n_groups=n_groups)
 
         caplog.set_level(logging.WARNING)
@@ -222,7 +238,8 @@ class ArgumentTests:
     @pytest.mark.parametrize("A_two_dim", [False, True])
     @pytest.mark.uncollect_if(func=is_invalid_transformation)
     def test_Y_df_bad_columns(self, transformX, transformA, A_two_dim):
-        gs = GridSearch(self.estimator, self.disparity_criterion)
+        gs = GridSearch(self.estimator, self.disparity_criterion,
+                        sample_weight_name=self.sample_weight_name)
         X, Y, A = _quick_data(A_two_dim)
 
         Y_two_col_df = pd.DataFrame({"a": Y, "b": Y})
@@ -230,15 +247,15 @@ class ArgumentTests:
             gs.fit(transformX(X),
                    Y_two_col_df,
                    sensitive_features=transformA(A))
-
-        assert "bad input shape" in execInfo.value.args[0]
+        assert get_sklearn_expected_1d_message() in execInfo.value.args[0]
 
     @pytest.mark.parametrize("transformA", candidate_A_transforms)
     @pytest.mark.parametrize("transformX", candidate_X_transforms)
     @pytest.mark.parametrize("A_two_dim", [False, True])
     @pytest.mark.uncollect_if(func=is_invalid_transformation)
     def test_Y_ndarray_bad_columns(self, transformX, transformA, A_two_dim):
-        gs = GridSearch(self.estimator, self.disparity_criterion)
+        gs = GridSearch(self.estimator, self.disparity_criterion,
+                        sample_weight_name=self.sample_weight_name)
         X, Y, A = _quick_data(A_two_dim)
 
         Y_two_col_ndarray = np.stack((Y, Y), -1)
@@ -246,28 +263,29 @@ class ArgumentTests:
             gs.fit(transformX(X),
                    Y_two_col_ndarray,
                    sensitive_features=transformA(A))
-
-        assert "bad input shape" in execInfo.value.args[0]
+        assert get_sklearn_expected_1d_message() in execInfo.value.args[0]
 
     # ----------------------------
 
     def test_no_predict_before_fit(self):
-        gs = GridSearch(self.estimator, self.disparity_criterion)
+        gs = GridSearch(self.estimator, self.disparity_criterion,
+                        sample_weight_name=self.sample_weight_name)
         X, _, _ = _quick_data()
 
         with pytest.raises(NotFittedError) as execInfo:
             gs.predict(X)
 
-        assert _NO_PREDICT_BEFORE_FIT == execInfo.value.args[0]
+        assert not_fitted_error_msg.format(GridSearch.__name__) == execInfo.value.args[0]
 
     def test_no_predict_proba_before_fit(self):
-        gs = GridSearch(self.estimator, self.disparity_criterion)
+        gs = GridSearch(self.estimator, self.disparity_criterion,
+                        sample_weight_name=self.sample_weight_name)
         X, _, _ = _quick_data()
 
         with pytest.raises(NotFittedError) as execInfo:
             gs.predict_proba(X)
 
-        assert _NO_PREDICT_BEFORE_FIT == execInfo.value.args[0]
+        assert not_fitted_error_msg.format(GridSearch.__name__) == execInfo.value.args[0]
 
 
 # Tests specific to Classification
@@ -278,7 +296,8 @@ class ConditionalOpportunityTests(ArgumentTests):
     @pytest.mark.parametrize("A_two_dim", [False, True])
     @pytest.mark.uncollect_if(func=is_invalid_transformation)
     def test_Y_ternary(self, transformX, transformY, transformA, A_two_dim):
-        gs = GridSearch(self.estimator, self.disparity_criterion)
+        gs = GridSearch(self.estimator, self.disparity_criterion,
+                        sample_weight_name=self.sample_weight_name)
         X, Y, A = _quick_data(A_two_dim)
         Y[0] = 0
         Y[1] = 1
@@ -297,7 +316,8 @@ class ConditionalOpportunityTests(ArgumentTests):
     @pytest.mark.parametrize("A_two_dim", [False, True])
     @pytest.mark.uncollect_if(func=is_invalid_transformation)
     def test_Y_not_0_1(self, transformX, transformY, transformA, A_two_dim):
-        gs = GridSearch(self.estimator, self.disparity_criterion)
+        gs = GridSearch(self.estimator, self.disparity_criterion,
+                        sample_weight_name=self.sample_weight_name)
         X, Y, A = _quick_data(A_two_dim)
         Y = Y + 1
 
@@ -309,11 +329,21 @@ class ConditionalOpportunityTests(ArgumentTests):
         assert _LABELS_NOT_0_1_ERROR_MESSAGE == execInfo.value.args[0]
 
 
+# Set up Pipeline estimator
+class TestPipelineEstimator(ConditionalOpportunityTests):
+    def setup_method(self, method):
+        self.estimator = Pipeline([('scaler', StandardScaler()),
+                                   ('logistic', LogisticRegression(solver='liblinear'))])
+        self.disparity_criterion = DemographicParity()
+        self.sample_weight_name = 'logistic__sample_weight'
+
+
 # Set up DemographicParity
 class TestDemographicParity(ConditionalOpportunityTests):
     def setup_method(self, method):
         self.estimator = LogisticRegression(solver='liblinear')
         self.disparity_criterion = DemographicParity()
+        self.sample_weight_name = 'sample_weight'
 
 
 # Test EqualizedOdds
@@ -321,10 +351,13 @@ class TestEqualizedOdds(ConditionalOpportunityTests):
     def setup_method(self, method):
         self.estimator = LogisticRegression(solver='liblinear')
         self.disparity_criterion = EqualizedOdds()
+        self.sample_weight_name = 'sample_weight'
 
 
 # Tests specific to BoundedGroupLoss
 class TestBoundedGroupLoss(ArgumentTests):
     def setup_method(self, method):
         self.estimator = LinearRegression()
-        self.disparity_criterion = GroupLossMoment(ZeroOneLoss())
+        eps = 0.01
+        self.disparity_criterion = BoundedGroupLoss(ZeroOneLoss(), upper_bound=eps)
+        self.sample_weight_name = 'sample_weight'
